package com.nesimtunc.imageloader;


import java.io.BufferedReader;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FilterInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.lang.ref.WeakReference;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Queue;

import org.apache.http.client.ClientProtocolException;

import tr.com.turkcell.connected.car.R;
import android.graphics.Bitmap;
import android.graphics.Bitmap.Config;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.Paint;
import android.graphics.PorterDuff.Mode;
import android.graphics.PorterDuffXfermode;
import android.graphics.Rect;
import android.graphics.RectF;
import android.graphics.drawable.BitmapDrawable;
import android.graphics.drawable.Drawable;
import android.os.Handler;
import android.view.View;
import android.widget.ImageView;
import android.widget.ProgressBar;

public class ImageLoader {
	static private ImageLoader _instance;
	private static Boolean _showIndicator;

	private static long CACHE_TIMEOUT = 1000 * 60 * 60 * 24; // 1 day

	static public ImageLoader getInstance() {
		if (_instance == null) {
			_instance = new ImageLoader();
		}
		_showIndicator = false;
		return _instance;
	}

	static public ImageLoader getInstance(Boolean showIndicator) {
		if (_instance == null) {
			_instance = new ImageLoader();
		}
		_showIndicator = showIndicator;
		return _instance;
	}

	static public ImageLoader getInstance(Boolean showIndicator, long cacheTimeout) {
		if (_instance == null) {
			_instance = new ImageLoader();
		}
		_showIndicator = showIndicator;
		return _instance;
	}

	private HashMap<String, WeakReference<Drawable>> _urlToBitmap;
	private Queue<Group> _queue;
	private DownloadThread _thread;
	private Bitmap _missing;
	private boolean _busy;

	/**
	 * Constructor
	 */
	private ImageLoader() {
		_urlToBitmap = new HashMap<String, WeakReference<Drawable>>();
		_queue = new LinkedList<Group>();
		_busy = false;
	}

	public Bitmap get(String url) {
		String hashedKey = Utilities.getMD5(url);
		if (_urlToBitmap.containsKey(hashedKey)) {
			WeakReference<Drawable> d = _urlToBitmap.get(hashedKey);
			if (d == null) {
				_urlToBitmap.remove(hashedKey);
			} else {
				if (d.get() != null) {
					Bitmap bitmap = ((BitmapDrawable) d.get()).getBitmap();
					return bitmap;
				} else {
					_urlToBitmap.remove(hashedKey);
				}
			}
		}
		return null;
	}

	public void load(ImageView image, String url) {
		load(image, url, false);
	}

	public void load(ImageView image, String url, boolean cache) {
		if (get(url) != null) {
			if (image != null) {
				image.setImageBitmap(get(url));
			}
		} else {
			showProgressBar(image);
			queue(image, url, cache, false);
		}
	}

	public void load(ImageView image, String url, boolean cache, boolean rounded) {
		if (get(url) != null) {
			if (image != null) {
				image.setImageBitmap(get(url));
			}
		} else {
			image.setImageBitmap(null);
			queue(image, url, cache, rounded);
		}
	}

	public void queue(ImageView image, String url, boolean cache, boolean rounded) {
		Iterator<Group> it = _queue.iterator();
		if (image != null) {
			while (it.hasNext()) {
				if (it.next().equals(image)) {
					it.remove();
					break;
				}
			}
		} else if (url != null) {
			while (it.hasNext()) {
				if (it.next().equals(url)) {
					it.remove();
					break;
				}
			}
		}
		_queue.add(new Group(image, url, null, cache, rounded));
		loadNext();
	}

	public void clearQueue() {
		_queue = new LinkedList<Group>();
	}

	public void clearCache() {
		_urlToBitmap = new HashMap<String, WeakReference<Drawable>>();
	}

	public void cancel() {
		clearQueue();
		if (_thread != null) {
			_thread.disconnect();
			_thread = null;
		}
	}

	public void setMissingBitmap(Bitmap bitmap) {
		_missing = bitmap;
	}

	private void showProgressBar(ImageView image) {
		if (!_showIndicator) return;

		View v = (View) image.getParent();
		View pv = v.findViewById(R.id.progressBar);
		if (pv != null && pv instanceof ProgressBar) {
			ProgressBar p = (ProgressBar) pv;
			p.setIndeterminate(true);
			p.setVisibility(View.VISIBLE);
		}

		/*
		 * ViewGroup parentView = (ViewGroup) image.getParent(); int childNo =
		 * findChildNo(parentView, image); if (parentView != null) { Context
		 * context = image.getContext(); LinearLayout l = new
		 * LinearLayout(context); l.setId(mLinearLayoutId);
		 * l.setLayoutParams(new LayoutParams(image.getLayoutParams()));
		 * l.setGravity(Gravity.CENTER); ProgressBar p = new
		 * ProgressBar(context); p.setId(mProgressId); l.addView(p);
		 * parentView.removeView(image); image.setTag(childNo);
		 * l.addView(image); parentView.addView(l, childNo); }
		 */
	}

	/*
	 * private int findChildNo(ViewGroup parentView, ImageView image) { int
	 * indexNo = -1; for (int i = 0; i < parentView.getChildCount(); i++) { View
	 * v = parentView.getChildAt(i); if (v instanceof ImageView && (ImageView) v
	 * == image) { indexNo = i; break; } } return indexNo; }
	 */

	private void removeProgressBar(Group group) {
		if (!_showIndicator) return;
		/*
		 * Integer childNo = (Integer) group.image.getTag(); View v = (View)
		 * group.image.getParent(); ViewGroup parentView = (ViewGroup)
		 * group.image.getParent(); LinearLayout l = (LinearLayout)
		 * v.findViewById(mLinearLayoutId);
		 * group.image.setLayoutParams(l.getLayoutParams());
		 * l.removeView(group.image); parentView.removeView(l);
		 * parentView.addView(group.image, childNo);
		 */

		View v = (View) group.image.getParent();
		if (v != null) {
			View pv = v.findViewById(R.id.progressBar);
			if (pv != null && pv instanceof ProgressBar) {
				ProgressBar p = (ProgressBar) pv;
				p.setIndeterminate(false);
				p.setVisibility(View.GONE);
			}
		}
	}

	private void loadNext() {
		Iterator<Group> it = _queue.iterator();
		if (!_busy && it.hasNext()) {
			_busy = true;
			Group group = (Group) it.next();
			it.remove();
			// double check image availability
			if (get(group.url) != null) {
				if (group.image != null) {
					removeProgressBar(group);
					group.image.setImageBitmap(get(group.url));
				}
				_busy = false;
				loadNext();
			} else {
				_thread = new DownloadThread(group);
				_thread.start();
			}
		}
	}

	private void onLoad() {
		if (_thread != null) {
			Group group = _thread.group;
			if (group.bitmap != null) {
				if (group.cache) {
					Drawable d = new BitmapDrawable(group.bitmap);
					String hashedKey = Utilities.getMD5(group.url);
					_urlToBitmap.put(hashedKey, new WeakReference<Drawable>(d));
				}
				if (group.image != null) {
					removeProgressBar(group);
					group.image.setImageBitmap(group.bitmap);
				}
			} else if (_missing != null) {
				if (group.image != null) {
					removeProgressBar(group);
					group.image.setImageBitmap(_missing);
				}
			}
		}
		_thread = null;
		_busy = false;
		loadNext();
	}

	public class Group {
		public Group(ImageView image, String url, Bitmap bitmap, boolean cache) {
			this.image = image;
			this.url = url;
			this.bitmap = bitmap;
			this.cache = cache;
			this.rounded = false;
		}

		public Group(ImageView image, String url, Bitmap bitmap, boolean cache, boolean rounded) {
			this.image = image;
			this.url = url;
			this.bitmap = bitmap;
			this.cache = cache;
			this.rounded = rounded;
		}

		public ImageView image;
		public String url;
		public Bitmap bitmap;
		public boolean cache;
		public boolean rounded;
	}

	private class DownloadThread extends Thread {
		final Handler threadHandler = new Handler();
		final Runnable threadCallback = new Runnable() {
			public void run() {
				onLoad();
			}
		};
		private HttpURLConnection _conn;
		public Group group;

		public DownloadThread(Group group) {
			this.group = group;
		}

		@Override
		public void run() {
			InputStream inStream = null;
			_conn = null;
			try {
				String hashedKey = Utilities.getMD5(group.url);
				Bitmap bitMap = get(group.url);
				if (bitMap == null) {
					File f = new File(Utilities.CACHE_DIRECTORY, hashedKey);
					if (f != null && f.exists()) {
						boolean timeout = false;
						try {
							long cachetimeout = CACHE_TIMEOUT;
							timeout = f.lastModified() + cachetimeout < new Date().getTime();
						} catch (Exception e1) {
							// TODO Auto-generated catch block
							e1.printStackTrace();
						}
						if (timeout) {
							try {
								f.delete();
							} catch (Exception e) {
							}
						}
					}
					if (!f.exists()) {
						_conn = (HttpURLConnection) new URL(group.url).openConnection();
						_conn.setDoInput(true);
						_conn.connect();
						inStream = _conn.getInputStream();
						if (f.createNewFile()) {
							FileOutputStream fo = new FileOutputStream(f);
							byte[] buffer = new byte[256];
							int size;
							while ((size = inStream.read(buffer)) > 0) {
								fo.write(buffer, 0, size);
							}
							fo.flush();
							fo.close();
						}
						//Log.i("IMG_DOWNLOADED", group.url);
						inStream.close();
						_conn.disconnect();
						inStream = null;
						_conn = null;
					}
					bitMap = BitmapFactory.decodeFile(f.getPath());
				}
				if (group.rounded) {
					group.bitmap = getRoundedCornerBitmap(bitMap);
				} else {
					group.bitmap = bitMap;
				}
			} catch (Exception ex) {
				//Log.i("IMG_DOWNLOAD_FAILED", group.url);
				ex.printStackTrace();
			} finally {
				threadHandler.post(threadCallback);
			}
		}

		public void disconnect() {
			if (_conn != null) {
				_conn.disconnect();
			}
		}
	}

	static class FlushedInputStream extends FilterInputStream {
		public FlushedInputStream(InputStream inputStream) {
			super(inputStream);
		}

		@Override
		public long skip(long n) throws IOException {
			long totalBytesSkipped = 0L;
			while (totalBytesSkipped < n) {
				long bytesSkipped = in.skip(n - totalBytesSkipped);
				if (bytesSkipped == 0L) {
					int b = read();
					if (b < 0) {
						break; // we reached EOF
					} else {
						bytesSkipped = 1; // we read one byte
					}
				}
				totalBytesSkipped += bytesSkipped;
			}
			return totalBytesSkipped;
		}
	}

	public boolean reSaveImage(String imageResaveUrl) {
		try {

			URL imageServer = new URL(imageResaveUrl);
			HttpURLConnection tc = (HttpURLConnection) imageServer.openConnection();
			InputStream inputStream = tc.getInputStream();
			BufferedReader r = new BufferedReader(new InputStreamReader(inputStream));
			StringBuilder serverResponse = new StringBuilder();
			String line;
			while ((line = r.readLine()) != null) {
				serverResponse.append(line);
			}
			tc.disconnect();
			tc = null;
			return serverResponse.toString().equals("succeed");
		} catch (ClientProtocolException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return false;

	}

	public Bitmap getRoundedCornerBitmap(Bitmap bitmap) {
		Bitmap output = Bitmap.createBitmap(bitmap.getWidth(), bitmap.getHeight(), Config.ARGB_8888);
		Canvas canvas = new Canvas(output);

		final int color = 0xff424242;
		final Paint paint = new Paint();
		final Rect rect = new Rect(0, 0, bitmap.getWidth(), bitmap.getHeight());
		final RectF rectF = new RectF(rect);
		final float roundPx = 5;

		paint.setAntiAlias(true);
		canvas.drawARGB(0, 0, 0, 0);
		paint.setColor(color);
		canvas.drawRoundRect(rectF, roundPx, roundPx, paint);

		paint.setXfermode(new PorterDuffXfermode(Mode.SRC_IN));
		canvas.drawBitmap(bitmap, rect, rect, paint);

		return output;
	}

}
